#!/usr/bin/env python
#python 3

## Fabian Ehrentraud, TU Wien, 2010

from urllib.request import *
import re

class Question:
	def __init__(self, category, question, occurrences):
		self.category = category.strip()
		self.question = question.strip()
		self.occurrences = occurrences.strip()
	def toString(self):
		return self.category + " - " + self.question + " - " + self.occurrences
	def contains(self, str):
		#if self contains str in occurrences then return times of occurrence
		return len(re.findall(str, self.occurrences))
	def get_contains(self, year):
		return re.findall(year+"/.|"+year+"-[^/]*", self.occurrences)

class QColl:
	def __init__(self):
		self.qs = []
	def toString(self):
		x = ""
		for q in self.qs:
			x += q.toString() + "\n\n"
		return x
	def add(self, q):
		self.qs.append(q)
	def has_not_and(self, strl):
		qn = QColl()
		for q in self.qs:
			f = True
			for str in strl:
				if q.contains(str):
					f = False
			if f == True:
				qn.add(q)
		return qn
	def has_and(self, strl):
		qn = QColl()
		for q in self.qs:
			f = True
			for str in strl:
				if not q.contains(str):
					f = False
			if f == True:
				qn.add(q)
		return qn
	def has_or(self, strl):
		qn = QColl()
		for q in self.qs:
			f = False
			for str in strl:
				if q.contains(str):
					f = True
			if f == True:
				qn.add(q)
		return qn
	def count(self):
		return len(self.qs)
	def occurr_more_often_than(self, strl, countmin):
		qn = QColl()
		for q in self.qs:
			count = 0
			for str in strl:
				count += q.contains(str)
			if count>countmin:
				qn.add(q)
		return qn
	def invert(self, universe):
		qn = QColl()
		for q in universe.qs:
			if q not in self.qs:
				qn.add(q)
		return qn
	def dates(self, yearl):
		dl = []
		for q in self.qs:
			for year in yearl:
				for qf in q.get_contains(year):
					if qf not in dl:
						dl.append(qf)
		#dl.sort(lambda x, y: x - y) #TODO: sort
		return dl

"""Let the fun begin!"""

ur = urlopen("http://vowi.fsinf.at/wiki?title=TU_Wien:Datenkommunikation_VO_%28van_As%29/PO_Fragen_WS09")
fullc = ur.read()

#fo = open("test.txt", "w")

cat = ""
qs = QColl()

#with a clean wiki-code this regex works, if there are some extra linebreaks etc, it may have to be adjusted
for m in re.finditer("(class=\"mw-headline\"> *(?P<category>[^<]*)<[^\n]*\n[^\n]*)?class=\"mw-headline\"> *(?P<title>[^<]*)<[^\n]*\n([^\n]*<b>(?P<occurrences>[^<]*)</b>)?",fullc.decode()):
	if m.group("category") != None:
		cat = m.group("category")
		#fo.write(m.group("category")+"\n\n")
	#fo.write(m.group("title")+'\n'+m.group("occurrences")+"\n\n")
	#fo.write(m.group("title")+"\n")
	qs.add(Question(cat, m.group("title"), m.group("occurrences")))
	#print(qs.toString())

#fo.close()

print("Insgesamt %s Fragen im VoWi" %(qs.count()))

f89or = qs.has_or(["2009","2008"])
count89or = f89or.count()
f89and = qs.has_and(["2009","2008"])
count89and = f89and.count()

print("Termine in 2008 und 2009: %s" %(qs.dates(["2008","2009"])))
print("2008 und 2009 zusammen = %s Fragen" %(count89or))

f89multi = f89or.occurr_more_often_than(["2008", "2009"], 1)
print("%2.2f%% der Fragen aus [2008, 2009] sind öfter als 1x in [2008, 2009] gestellt worden" %(100*f89multi.count()/count89or))
#print(f89multi.toString())
#print(f89multi.invert(qs).toString())

only6 = f89or.has_and(["2006"])
#print(only6.toString())
print("Die Fragen von 2008 und 2009 finden sich zu %2.2f%% in [2006]" %(100*only6.count()/count89or))

only5 = f89or.has_and(["2005"])
print("Die Fragen von 2008 und 2009 finden sich zu %2.2f%% in [2005]" %(100*only5.count()/count89or))

f6or5 = f89or.has_or(["2006","2005"])
print("Die Fragen von 2008 und 2009 finden sich zu %2.2f%% in [2006, 2005]" %(100*f6or5.count()/count89or))

f5or4 = f89or.has_or(["2005","2004"])
print("Die Fragen von 2008 und 2009 finden sich zu %2.2f%% in [2005, 2004]" %(100*f5or4.count()/count89or))

f6or5or4 = f89or.has_or(["2006","2005","2004"])
print("Die Fragen von 2008 und 2009 finden sich zu %2.2f%% in [2006, 2005, 2004]" %(100*f6or5or4.count()/count89or))

f6or5or4or3 = f89or.has_or(["2006","2005","2004","2003"])
print("Die Fragen von 2008 und 2009 finden sich zu %2.2f%% in [2006, 2005, 2004, 2003]" %(100*f6or5or4or3.count()/count89or))

f6or5or4or3or2 = f89or.has_or(["2006","2005","2004","2003","2002"])
print("Die Fragen von 2008 und 2009 finden sich zu %2.2f%% in [2006, 2005, 2004, 2003, 2002]" %(100*f6or5or4or3or2.count()/count89or))

only6no54 = f89or.has_and(["2006"]).has_not_and(["2005","2004"])
print("Fragen von 2008 und 2009 die mit 2006 beantwortet werden könnten aber nicht mit 2005 oder 2004: %2.2f%%" %(100*only6no54.count()/count89or))

print("\nNicht durch 2006, 2005 und 2004 abgedeckt:\n")
print(f89or.has_not_and(["2006","2005","2004"]).toString())

print("Nicht durch das PO Skriptum abgedeckte Fragen:\n")
print(f89or.has_not_and(["2006","2005","2004","2003","2002"]).toString())